local ADDON, Package = ...

local Kernel = Package.Kernel
local Enum = {}

function Enum.clone(enumerable)
  local stash = {}

  for k, v in pairs(enumerable) do
    if Kernel.is_table(v) then
      stash[k] = Enum.clone(v)
    else
      stash[k] = v
    end
  end

  return stash
end

function Enum.merge(to, from, override)
  local stash = Enum.clone(to)

  for k, v in pairs(from) do
    if Kernel.is_table(v) and Kernel.is_table(stash[k]) then
      stash[k] = Enum.merge(stash[k], v, override)
    else
      if stash[k] == nil or override ~= false then
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.omit(enumerable, keys, depth)
  local stash = {}
  local depth = depth or 0

  for k, v in pairs(enumerable) do
    if not Enum.is_member(keys, k) then
      if Kernel.is_table(v) and depth > 0 then
        stash[k] = Enum.omit(v, keys, depth - 1)
      else
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.rename(enumerable, from, to)
  local stash = {}

  for k, v in pairs(enumerable) do
    local i = k == from and to or k

    if Kernel.is_table(v) then
      stash[i] = Enum.rename(v, from, to)
    else
      stash[i] = v
    end
  end

  return stash
end

function Enum.filter(enumerable, value)
  local stash = {}

  for _, entry in pairs(enumerable) do
    if entry ~= value then
      table.insert(stash, entry)
    end
  end

  return stash
end

function Enum.is_member(enumerable, value)
  for _, entry in pairs(enumerable) do
    if entry == value then
      return true
    end
  end

  return false
end

function Enum.concat(left, right)
  local stash = Enum.clone(left)

  for _, entry in pairs(right) do
    table.insert(stash, entry)
  end

  return stash
end

Package.Enum = Enum