local ADDON, Package = ...

local Module = {}
local Registry = {}

local PAPER_DOLL_ITEMS = {
  HeadSlot = "CharacterHeadSlot",
  NeckSlot = "CharacterNeckSlot",
  ShoulderSlot = "CharacterShoulderSlot",
  BackSlot = "CharacterBackSlot",
  ChestSlot = "CharacterChestSlot",
  ShirtSlot = "CharacterShirtSlot",
  TabardSlot = "CharacterTabardSlot",
  WristSlot = "CharacterWristSlot",
  HandsSlot = "CharacterHandsSlot",
  WaistSlot = "CharacterWaistSlot",
  LegsSlot = "CharacterLegsSlot",
  FeetSlot = "CharacterFeetSlot",
  Finger0Slot = "CharacterFinger0Slot",
  Finger1Slot = "CharacterFinger1Slot",
  Trinket0Slot = "CharacterTrinket0Slot",
  Trinket1Slot = "CharacterTrinket1Slot",
  MainHandSlot = "CharacterMainHandSlot",
  SecondaryHandSlot = "CharacterSecondaryHandSlot",
  RangedSlot = "CharacterRangedSlot",
  AmmoSlot = "CharacterAmmoSlot"
}

local INSPECT_PAPER_DOLL_ITEMS = {
  HeadSlot = "InspectHeadSlot",
  NeckSlot = "InspectNeckSlot",
  ShoulderSlot = "InspectShoulderSlot",
  BackSlot = "InspectBackSlot",
  ChestSlot = "InspectChestSlot",
  ShirtSlot = "InspectShirtSlot",
  TabardSlot = "InspectTabardSlot",
  WristSlot = "InspectWristSlot",
  HandsSlot = "InspectHandsSlot",
  WaistSlot = "InspectWaistSlot",
  LegsSlot = "InspectLegsSlot",
  FeetSlot = "InspectFeetSlot",
  Finger0Slot = "InspectFinger0Slot",
  Finger1Slot = "InspectFinger1Slot",
  Trinket0Slot = "InspectTrinket0Slot",
  Trinket1Slot = "InspectTrinket1Slot",
  MainHandSlot = "InspectMainHandSlot",
  SecondaryHandSlot = "InspectSecondaryHandSlot",
  RangedSlot = "InspectRangedSlot"
}

local TRADE_ITEMS = {
  "TradePlayerItem1ItemButton",
  "TradePlayerItem2ItemButton",
  "TradePlayerItem3ItemButton",
  "TradePlayerItem4ItemButton",
  "TradePlayerItem5ItemButton",
  "TradePlayerItem6ItemButton",
  "TradePlayerItem7ItemButton",
  "TradeRecipientItem1ItemButton",
  "TradeRecipientItem2ItemButton",
  "TradeRecipientItem3ItemButton",
  "TradeRecipientItem4ItemButton",
  "TradeRecipientItem5ItemButton",
  "TradeRecipientItem6ItemButton",
  "TradeRecipientItem7ItemButton"
}

local agent = CreateFrame("Frame")

function Module.create_border(frame, width, height)
  if frame and not frame.ItemQualityBorder then
    local border = frame:CreateTexture(nil, "OVERLAY", nil, -1)

    border:SetSize(width, height)
    border:SetBlendMode("ADD")
    border:SetTexture(130739)
    border:SetPoint("CENTER")
    border:Hide()

    frame.ItemQualityBorder = border
  end
end

function Module.update_border(frame, item_quality)
  if frame then
    if frame.ItemQualityBorder and item_quality then
      local item_quality_color = ITEM_QUALITY_COLORS[item_quality]

      frame.ItemQualityBorder:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
      frame.ItemQualityBorder:Show()
    else
      frame.ItemQualityBorder:Hide()
    end
  end
end

function Module.create_border_on_equipped_items(frames)
  for slot_name, frame_name in pairs(frames) do
    local width = 0
    local height = 0

    if slot_name == "AmmoSlot" then
      width = 47
      height = 47
    else
      width = 65
      height = 65
    end

    Module.create_border(_G[frame_name], width, height)
  end
end

function Module.update_border_on_equipped_items(frames, unit_id)
  for slot_name, frame_name in pairs(frames) do
    local id = GetInventorySlotInfo(slot_name)
    local item_quality = GetInventoryItemQuality(unit_id, id)

    Module.update_border(_G[frame_name], item_quality)
  end
end

function Module.create_border_on_trade_items()
  for at = 1, #TRADE_ITEMS do
    Module.create_border(_G[TRADE_ITEMS[at]], 65, 65)
  end
end

function Module.update_border_on_trade_items()
  for at = 1, MAX_TRADE_ITEMS do
    local player_item_name, _, _, player_item_quality = GetTradePlayerItemInfo(at)
    local target_item_name, _, _, target_item_quality = GetTradeTargetItemInfo(at)

    Module.update_border(_G[TRADE_ITEMS[at]], player_item_name and player_item_quality)
    Module.update_border(_G[TRADE_ITEMS[at + MAX_TRADE_ITEMS]], target_item_name and target_item_quality)
  end
end

function Module.enable()
  agent:SetScript("OnEvent", Module.on_event)
  agent:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
  agent:RegisterEvent("INSPECT_READY")
  agent:RegisterEvent("TRADE_SHOW")
  agent:RegisterEvent("TRADE_UPDATE")
  agent:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED")
  agent:RegisterEvent("TRADE_TARGET_ITEM_CHANGED")

  PaperDollFrame:HookScript("OnShow", function () Module.create_border_on_equipped_items(PAPER_DOLL_ITEMS) end)
  PaperDollFrame:HookScript("OnShow", function () Module.update_border_on_equipped_items(PAPER_DOLL_ITEMS, "player") end)
end

function Module.disable()
  agent:SetScript("OnEvent", nil)
  agent:UnregisterEvent("UPDATE_INVENTORY_DURABILITY")
  agent:UnregisterEvent("INSPECT_READY")
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

function Registry.UPDATE_INVENTORY_DURABILITY()
  if PaperDollFrame:IsVisible() then
    Module.create_border_on_equipped_items(PAPER_DOLL_ITEMS)
    Module.update_border_on_equipped_items(PAPER_DOLL_ITEMS, "player")
  end
end

function Registry.INSPECT_READY(inspectee_guid)
  Module.create_border_on_equipped_items(INSPECT_PAPER_DOLL_ITEMS)
  Module.update_border_on_equipped_items(INSPECT_PAPER_DOLL_ITEMS, "target")
end

function Registry.TRADE_SHOW()
  Module.create_border_on_trade_items()
  Module.update_border_on_trade_items()
end

function Registry.TRADE_UPDATE()
  Module.update_border_on_trade_items()
end

function Registry.TRADE_PLAYER_ITEM_CHANGED()
  Module.update_border_on_trade_items()
end

function Registry.TRADE_TARGET_ITEM_CHANGED()
  Module.update_border_on_trade_items()
end

Package.UseItemQualityColorBorder = Module